//
//  dis_analyzer.cpp
//  
//
//  Created by chyde on 8/26/21.
//  generic analyzer of output root file from Delphes
//  Delphes control card
//  ../delphes/cards/CORE/Delphes_CORE_35.tcl
//  Delphes output files in ../delphes/output/"fname".root
//

#include <stdio.h>

#include <TTree.h>
#include <TFile.h>
#include <TSystem.h>
#include <TCanvas.h>
#include <TH1.h>
#include <TH2D.h>
#include <TChain.h>
#include <TF1.h>
#include <TLorentzVector.h>
#include <TRandom3.h>
#include <TStyle.h>


#ifdef __CLING__
R__LOAD_LIBRARY(libDelphes)
#include "classes/DelphesClasses.h"
#include "external/ExRootAnalysis/ExRootProgressBar.h"
#include "external/ExRootAnalysis/ExRootTreeBranch.h"
#include "external/ExRootAnalysis/ExRootTreeReader.h"
#endif

int dis_analyzer(const char *fname="output18x275"){
    char inputfile[128];
    sprintf(inputfile,"output/%s.root",fname);
    printf("%s\n",inputfile);
    gSystem->Load("libDelphes");
    TRandom3 ran3;

    // Create chain of root trees
    TChain chain("Delphes");
    chain.Add(inputfile);

    // Create object of class ExRootTreeReader
    ExRootTreeReader *treeReader = new ExRootTreeReader(&chain);
    Long64_t numberOfEntries = treeReader->GetEntries();
    
    // Get pointers to branches created by Delphes control card
    TClonesArray *branchJet = treeReader->UseBranch("Jet");
    TClonesArray *branchElectron = treeReader->UseBranch("Electron");
    TClonesArray *branchParticle = treeReader->UseBranch("Particle");
    TClonesArray *branchPhoton = treeReader->UseBranch("Photon");
    TClonesArray *branchEFlow = treeReader->UseBranch("EFlowTrack");
    TClonesArray *branchTrack = treeReader->UseBranch("Track");
    
    GenParticle * thisParticle; // Generated  particle branch
    Photon * thisPhoton;        //  Identified photon branch
    Electron * thisElectron;    //  Identified electron branch
    Jet * thisJet;
    // Track * thisEFlow; //?? Ask Miguel Arratia
    Track * thisTrack;

    int nbins = 100;
    TH1 *h1_Jet_Mass = new TH1D("h1_Jet_Mass","Jet Mass; Mass (GeV) ;  Events",
                                nbins,0,100.);
    TH1 *h1_Jet_eta = new TH1D("h1_Jet_eta","Jet #eta; Pseudo Rapidituy ;  Events",
                                nbins,-5.0,5.0);
    TH1 *h1_Jet_pT = new TH1D("h1_Jet_pT","Jet P_{T}; P_{T} (GeV) ;  Events",
                                nbins,0,100.);
    TH2 *h2_Jet_neut_vs_char = new TH2D("h2_Jet_neut_vs_char",
                                        "Neutral vs Charge Multiplicity in Jet",
                                        20,0,20, 20,0.0,20.0);

   // Loop over all events
    Int_t entry=0;
    Int_t nElectron=0;
    Int_t nJet=0;
    Int_t nPhoton=0;
    for(entry = 0; entry < numberOfEntries; ++entry)
    {
        // Load selected branches with data from specified event
        treeReader->ReadEntry(entry);
        nJet += branchJet->GetEntries();
        for (int iJet=0; iJet<branchJet->GetEntries(); iJet++) {
            thisJet = (Jet *) branchJet->At(iJet);
            h1_Jet_Mass->Fill(thisJet->Mass);
            h1_Jet_eta->Fill(thisJet->Eta);
            h1_Jet_pT->Fill(thisJet->PT);
            h2_Jet_neut_vs_char->Fill(thisJet->NCharged,thisJet->NNeutrals);
        }
    }
    
    TCanvas *cJ = new TCanvas("cJ",fname,50,50,800,800);
    cJ->Divide(2,2);
    cJ->cd(1);
    h1_Jet_Mass->Draw();
    gPad->SetLogy();
    cJ->cd(2);
    h1_Jet_eta->Draw();
    gPad->SetLogy();
    cJ->cd(3);
    h1_Jet_pT->Draw();
    gPad->SetLogy();
    cJ->cd(4);
    h2_Jet_neut_vs_char->Draw();
    
    return entry;
}
